'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

require('rxjs/add/observable/fromPromise');
require('rxjs/add/operator/concatMap');
require('rxjs/add/operator/distinct');
require('rxjs/add/operator/finally');
require('rxjs/add/operator/map');
require('rxjs/add/operator/shareReplay');
var Blockchain = _interopDefault(require('neo-blockchain-impl'));
var Node = _interopDefault(require('neo-blockchain-node'));
var Observable = require('rxjs/Observable');
var neoBlockchainRpc = require('neo-blockchain-rpc');
var leveldown = _interopDefault(require('leveldown'));
var levelup = _interopDefault(require('levelup'));
var levelUpStorage = _interopDefault(require('neo-blockchain-levelup'));
var vm = _interopDefault(require('neo-blockchain-vm'));

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function step(key, arg) {
        try {
          var info = gen[key](arg);
          var value = info.value;
        } catch (error) {
          reject(error);
          return;
        }

        if (info.done) {
          resolve(value);
        } else {
          Promise.resolve(value).then(_next, _throw);
        }
      }

      function _next(value) {
        step("next", value);
      }

      function _throw(err) {
        step("throw", err);
      }

      _next();
    });
  };
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

var FullNode$1 =
/*#__PURE__*/
function () {
  function FullNode(_ref) {
    var log = _ref.log,
        createLogForContext = _ref.createLogForContext,
        createProfile = _ref.createProfile,
        settings = _ref.settings,
        environment = _ref.environment,
        options$ = _ref.options$,
        onError = _ref.onError,
        onCreateBlockchain = _ref.onCreateBlockchain;
    _classCallCheck(this, FullNode);
    this._log = log;
    this._createLogForContext = createLogForContext;
    this._createProfile = createProfile;
    this._settings = settings;
    this._environment = environment;
    this._options$ = options$;

    this._onError = onError || function () {};

    this._onCreateBlockchain = // eslint-disable-next-line
    onCreateBlockchain || function () {
      var _ref2 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee(blockchain) {
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      return function (_x) {
        return _ref2.apply(this, arguments);
      };
    }();

    this._subscription = null;
    this._rpcServer = null;
  }

  _createClass(FullNode, [{
    key: "start",
    value: function () {
      var _ref3 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee2() {
        var nodeAndBlockchain$;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                nodeAndBlockchain$ = this._getNodeAndBlockchain$();
                this._subscription = nodeAndBlockchain$.subscribe();

                this._startRPCServer(nodeAndBlockchain$);

              case 3:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function start() {
        return _ref3.apply(this, arguments);
      }

      return start;
    }()
  }, {
    key: "_getNodeAndBlockchain$",
    value: function _getNodeAndBlockchain$() {
      var _this = this;

      var dispose = function () {
        var _ref5 = _asyncToGenerator(
        /*#__PURE__*/
        regeneratorRuntime.mark(function _callee3(_ref4) {
          var storage, blockchain, node;
          return regeneratorRuntime.wrap(function _callee3$(_context3) {
            while (1) {
              switch (_context3.prev = _context3.next) {
                case 0:
                  storage = _ref4.storage, blockchain = _ref4.blockchain, node = _ref4.node;
                  _context3.prev = 1;
                  _context3.next = 4;
                  return node.stop();

                case 4:
                  _context3.next = 6;
                  return blockchain.stop();

                case 6:
                  _context3.next = 8;
                  return storage.close();

                case 8:
                  _context3.next = 16;
                  break;

                case 10:
                  _context3.prev = 10;
                  _context3.t0 = _context3["catch"](1);

                  _this._log({
                    event: 'FULL_NODE_DISPOSE_ERROR',
                    error: _context3.t0
                  });

                  _context3.next = 15;
                  return _this.stop();

                case 15:
                  _this._onError();

                case 16:
                case "end":
                  return _context3.stop();
              }
            }
          }, _callee3, this, [[1, 10]]);
        }));

        return function dispose(_x2) {
          return _ref5.apply(this, arguments);
        };
      }();

      var currentResources = null;
      return this._options$.map(function (options) {
        return options.node;
      }).distinct().concatMap(function (options) {
        return Observable.Observable.fromPromise(Promise.resolve().then(_asyncToGenerator(
        /*#__PURE__*/
        regeneratorRuntime.mark(function _callee4() {
          var storage, blockchain, node;
          return regeneratorRuntime.wrap(function _callee4$(_context4) {
            while (1) {
              switch (_context4.prev = _context4.next) {
                case 0:
                  if (!(currentResources != null)) {
                    _context4.next = 3;
                    break;
                  }

                  _context4.next = 3;
                  return dispose(currentResources);

                case 3:
                  storage = levelUpStorage({
                    db: levelup(leveldown(_this._environment.dataPath)),
                    context: {
                      messageMagic: _this._settings.messageMagic
                    }
                  });
                  _context4.next = 6;
                  return Blockchain.create({
                    settings: _this._settings,
                    storage: storage,
                    vm: vm,
                    log: _this._log
                  });

                case 6:
                  blockchain = _context4.sent;
                  _context4.next = 9;
                  return _this._onCreateBlockchain(blockchain);

                case 9:
                  node = new Node({
                    blockchain: blockchain,
                    seeds: options.seeds
                  });
                  node.start();
                  currentResources = {
                    storage: storage,
                    blockchain: blockchain,
                    node: node
                  };
                  return _context4.abrupt("return", {
                    node: node,
                    blockchain: blockchain
                  });

                case 13:
                case "end":
                  return _context4.stop();
              }
            }
          }, _callee4, this);
        }))));
      }).finally(function () {
        if (currentResources != null) {
          dispose(currentResources);
        }
      }).shareReplay(1);
    }
  }, {
    key: "_startRPCServer",
    value: function () {
      var _ref7 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee5(nodeAndBlockchain$) {
        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                this._rpcServer = new neoBlockchainRpc.RPCServer({
                  log: this._log,
                  createLogForContext: this._createLogForContext,
                  createProfile: this._createProfile,
                  blockchain$: nodeAndBlockchain$.map(function (_ref8) {
                    var blockchain = _ref8.blockchain;
                    return blockchain;
                  }),
                  node$: nodeAndBlockchain$.map(function (_ref9) {
                    var node = _ref9.node;
                    return node;
                  }),
                  environment: this._environment.rpc,
                  options$: this._options$.map(function (options) {
                    return options.rpc;
                  }).distinct(),
                  onError: this._onError
                });
                _context5.next = 3;
                return this._rpcServer.start();

              case 3:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function _startRPCServer(_x3) {
        return _ref7.apply(this, arguments);
      }

      return _startRPCServer;
    }()
  }, {
    key: "stop",
    value: function () {
      var _ref10 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee6() {
        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                if (!(this._rpcServer != null)) {
                  _context6.next = 4;
                  break;
                }

                _context6.next = 3;
                return this._rpcServer.stop();

              case 3:
                this._rpcServer = null;

              case 4:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function stop() {
        return _ref10.apply(this, arguments);
      }

      return stop;
    }()
  }]);
  return FullNode;
}();

module.exports = FullNode$1;
//# sourceMappingURL=index.js.map
